package org.mathcs.ibookmark;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;

import android.app.ProgressDialog;
import android.os.AsyncTask;

/*
 * AsyncTask enables proper and easy use of the UI thread. This class allows to perform 
 * background operations and publish results on the UI thread without having to manipulate 
 * threads and/or handlers. AsyncTask is designed to be a helper class and should ideally be 
 * used for short operations (a few seconds at the most.)
 * 
 * An asynchronous task is defined by a computation that runs on a background thread and whose 
 * result is published on the UI thread. An asynchronous task is defined by 3 generic types, 
 * called Params, Progress and Result, and 4 steps, called onPreExecute, doInBackground, 
 * onProgressUpdate and onPostExecute. You do not call these methods directly; instead, you call
 * on the "execute(inputlist)" method, which in turn calls 'doInBackground' immediately after
 * calling 'onPreExecute'. While the thread executes, 'onProgressUpdate' is called and when
 * the thread is done, 'onPostExecute' will be called.
 * 
 * AsyncTask must be subclassed to be used. The subclass will override at least one method 
 * (doInBackground(Params...)), and most often will override a second one (onPostExecute(Result).)
 * 
 * As is standard for an AsyncTask, it is defined with three generic parameters:
 * 
 *   1st parameter defines the input types
 *   2nd parameter defines the progress reporting type
 *   3rd parameter defines the return type
 * 
 * Our specific subclass iterates through the text file provided as input to the "execute" method
 * and populates a list of Bookmark items according to the data. The text file location is given
 * as a URL, hence the necessity of using asynchronous operation. It returns the number of Bookmark 
 * items processed. It then updates the view so that the user can see the bookmark items loaded.
 *  
 *  @version 1.0 (beta) (Oct 2013)
 *  @author Bert G. Wachsmuth (http://www.mathcs.org/)
 */

public class BookmarkLoader extends AsyncTask<String, Integer, Integer> 
{
	private ProgressDialog progress = null;
	private BookmarkAdapter adapter = null;
	private boolean canceled = false;
	private String error = "no error";
	
	// Initializes the adapter and progress dialog defined in the parent class
	public BookmarkLoader(BookmarkAdapter adapter, ProgressDialog progress)
	{
		super();
		this.adapter = adapter;
		this.progress = progress;
	}
	
	// Called by 'execute' after 'onPreExwecute' is done.
	@Override
	protected Integer doInBackground(String... urls)
	{
		try
		{
			URL url = new URL(urls[0]);
		
			BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));			
			String ver = in.readLine();
			if (ver.equals(BookmarkActivity.VERSION))
			{
				int size = Integer.parseInt(in.readLine());
				int i = 0;
				while ((i < size) && (!canceled))
				{
					adapter.add(new Bookmark(in));
					i++;
				}
			}
			else
				throw new Exception("Data file has wrong format. Contact help. ");
			
			return adapter.getCount();
		}
		catch (Exception ex)
		{
			error = ex.toString();
			return -1;
		}
	}


    /*
     * Causes the thread to fall through and terminate by properly reporting
     * the number of items loaded so far.
     */
    public void cancelLoading()
    {
    	canceled = true;
    }
    
    // Start progress dialog and clear out any left-over items in the adapter
    protected void onPreExecute()
    {
    	progress.show();
		adapter.removeAll();
    }
    
    // When thread is done, check on the number of items generated as reported
    // in 'result' and act accordingly. Dismisses the progress dialog, notifies
    // adapter to update its view and then starts the image loading thread.
    protected void onPostExecute(Integer result) 
    {
    	if (result < 0)
    		adapter.add(new Bookmark("Sorry, bookmark file not avilable.", "Error: " + error));
    	else if (result == 0)
    		adapter.add(new Bookmark("No bookmarks available at the moment.", ""));
    	progress.dismiss();
		adapter.notifyDataSetChanged();
    }
}
